package com.example;

import javafx.application.Application;
import javafx.event.EventHandler;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.scene.Node;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.ContextMenu;
import javafx.scene.control.Label;
import javafx.scene.control.MenuItem;
import javafx.scene.input.MouseEvent;
import javafx.stage.Stage;
import javafx.stage.WindowEvent;

public class SampleApp extends Application {

	public static void main(String[] args) {
		launch(args);
	}

	@FXML Label lblSample;
	@FXML MenuItem menuItemOpen;

	@Override
	public void start(Stage primaryStage) throws Exception {
		primaryStage.setTitle("JavaFX でコンテキスト・メニューの振る舞いを修正する");

		FXMLLoader loader = new FXMLLoader(getClass().getResource("SampleApp.fxml"));
		loader.setController(this);
		Parent root = loader.load();
		Scene scene = new Scene(root);
		primaryStage.setScene(scene);
		primaryStage.show();

		// lblSampleのコンテキストメニューの振る舞いを修正します。
		fix(lblSample.getContextMenu(), true);

		// メニューバーの振る舞いを修正します。
		fix(menuItemOpen.getParentPopup(), false);
	}

	private static void fix(ContextMenu contextMenu, boolean hideOnMouseReleased) {
		contextMenu.addEventHandler(WindowEvent.WINDOW_SHOWING, new EventHandler<WindowEvent>() {
			@Override
			public void handle(WindowEvent windowEvent) {
				Node contextMenuNode = contextMenu.getStyleableNode();
				for (MenuItem menuItem : contextMenu.getItems()) {
					Node menuItemNode = menuItem.getStyleableNode();
					if (menuItemNode == null) {
						continue;
					}

					// マウスカーソルがメニューアイテムの外側に出てとき、
					// 他ノードにフォーカス要求を出すことでメニューアイテムからフォーカスを外します。
					// ここでは他ノードとして親であるcontextMenuNodeを指定してます。（他の適当なノードでもOK）
					menuItemNode.addEventHandler(MouseEvent.MOUSE_EXITED, mouseEvent -> {
						contextMenuNode.requestFocus();
					});

					// メニューアイテムで押下されたマウスボタンが離されたとき、
					// メニューアイテムがフォーカスされていなければ、マウスイベントを消費します。
					// これによってメニューアイテムのアクション発動を抑止することができます。
					// また、hideOnMouseReleased の指定に従ってコンテキスト・メニューを非表示にします。
					menuItemNode.addEventFilter(MouseEvent.MOUSE_RELEASED, mouseEvent -> {
						if (!menuItemNode.isFocused()) {
							mouseEvent.consume();
							if (hideOnMouseReleased) {
								contextMenu.hide();
							}
						}
					});
				}
				contextMenu.removeEventHandler(WindowEvent.WINDOW_SHOWING,this);
			}
		});
	}
}
